package com.massivecraft.factions.cmd;

import com.massivecraft.factions.Perm;
import com.massivecraft.factions.cmd.type.TypeFaction;
import com.massivecraft.factions.entity.Faction;
import com.massivecraft.factions.entity.FactionColl;
import com.massivecraft.massivecore.MassiveException;
import com.massivecraft.massivecore.command.requirement.RequirementHasPerm;
import com.massivecraft.massivecore.command.type.TypeWorld;
import com.massivecraft.massivecore.command.type.primitive.TypeDouble;
import com.massivecraft.massivecore.ps.PS;
import org.bukkit.Location;
import org.bukkit.World;

import java.util.Collections;
import java.util.Set;

public class CmdFactionsSetOneLocation extends FactionsCommand {
    // -------------------------------------------- //
    // CONSTRUCT
    // -------------------------------------------- //

    private final boolean claim;

    public CmdFactionsSetOneLocation(boolean claim) {
        // Super
        super();

        this.setSetupEnabled(false);
        this.claim = claim;

        // Aliases
        this.addAliases("onel");

        this.addParameter(TypeWorld.get(), true, "world").setDesc("the world of the location");
        this.addParameter(TypeDouble.get(), true, "x").setDesc("the x coordinate of the location");
        this.addParameter(TypeDouble.get(), true, "y").setDesc("the y coordinate of the location");
        this.addParameter(TypeDouble.get(), true, "z").setDesc("the z coordinate of the location");
        this.addParameter(TypeFaction.get(), "faction").setDesc("you");

        // Requirements
        Perm perm = claim ? Perm.CLAIM_ONE : Perm.UNCLAIM_ONE;
        this.addRequirements(RequirementHasPerm.get(perm));
    }

    @Override
    public void perform() throws MassiveException {
        // Args
        World world = this.readArgAt(0);
        double x = this.readArgAt(1);
        double y = this.readArgAt(2);
        double z = this.readArgAt(3);
        final Faction newFaction = this.readArgAt(4);

        Set<PS> chunks = Collections.singleton(PS.valueOf(new Location(world, x, y, z)).getChunk());

        // Apply / Inform
        if (this.claim) {
            msender.tryClaim(newFaction, chunks);
        } else {
            msender.tryClaim(FactionColl.get().getNone(), chunks);
        }
    }

}
